__all__ = ["NrqlLexer"]


import re

from pygments.lexer import RegexLexer, words
from pygments.token import (
    Comment,
    Keyword,
    Name,
    Number,
    Operator,
    Punctuation,
    String,
    Text,
)

from .builtins import FUNCTIONS, KEYWORDS, OPERATORS, WORD_OPERATORS


class NrqlLexer(RegexLexer):
    name = "NRQL"
    aliases = [
        "nrql",
    ]
    mimetypes = ["text/x-nrql"]

    flags = re.IGNORECASE
    tokens = {
        "root": [
            (
                r"\s+",
                Text,
            ),
            (
                r"--.*?\n",
                Comment.Single,
            ),
            (
                r"//.*?\n",
                Comment.Single,
            ),
            (
                r"/\*",
                Comment.Multiline,
                "multiline-comments",
            ),
            (
                r"(" + "|".join(KEYWORDS) + r")\b",
                Keyword,
            ),
            (
                r"(" + "|".join(FUNCTIONS) + r")\b",
                Name.Function,
            ),
            (
                # r"(" + "|".join(OPERATORS) + r")\b",
                words(OPERATORS),
                Operator,
            ),
            (
                r"(" + "|".join(WORD_OPERATORS) + r")\b",
                Operator.Word,
            ),
            (
                r"([0-9]*\.[0-9]*|[0-9]+)(e[+-]?[0-9]+)?",
                Number.Float,
            ),
            (
                r"[0-9]+",
                Number.Integer,
            ),
            (
                r"'(''|[^'])*'",
                String.Single,
            ),
            (
                r"r'(''|[^'])*'",
                String.Regex,
            ),
            (
                r"`([^`])*`",
                String.Backtick,
            ),
            (
                r"[a-zA-Z]+([a-zA-Z0-9_\.]*)([a-zA-Z0-9_]*)",
                Name,
            ),
            (
                r"`[a-zA-Z0-9:_]+([a-zA-Z0-9:_\.]*)([a-zA-Z0-9:_]*)`",
                Name,
            ),
            (
                r"[;:()\[\],\.]",
                Punctuation,
            ),
        ],
        "multiline-comments": [
            (r"/\*", Comment.Multiline, "multiline-comments"),
            (r"\*/", Comment.Multiline, "#pop"),
            (r"[^/\*]+", Comment.Multiline),
            (r"[/*]", Comment.Multiline),
        ],
    }



__all__ = [
    "REQUIRED_KEYWORDS",
    "KEYWORDS",
    "EXTENDED_KEYWORDS",
    "FUNCTIONS",
    "OPERATORS",
    "WORD_OPERATORS",
]


REQUIRED_KEYWORDS = [
    "FROM",
    "SELECT",
]


KEYWORDS = [
    *REQUIRED_KEYWORDS,
    # Keywords: Constants
    "NULL",
    "TRUE",
    "FALSE",
    # Keywords: time operators
    "THIS",
    "LAST",
    "AT",
    "AGO",
    # Keywords: relative time and days
    "NOW",
    "TODAY",
    "YESTERDAY",
    # Keywords: day names
    "MONDAY",
    "TUESDAY",
    "WEDNESDAY",
    "THURSDAY",
    "FRIDAY",
    "SATURDAY",
    "SUNDAY",
    # Keywords: time units
    "MILLISECOND",
    "MILLISECONDS",
    "SECOND",
    "SECONDS",
    "MINUTE",
    "MINUTES",
    "HOUR",
    "HOURS",
    "DAY",
    "DAYS",
    "WEEK",
    "WEEKS",
    "MONTH",
    "MONTHS",
    "QUARTER",
    "QUARTERS",
    "YEAR",
    "YEARS",
    # Keywords: clause tokens
    "SHOW",
    "EVENT",
    "TYPES",
    "DELETE",
    "FROM",
    "WITH",
    "INNER",
    "LEFT",
    "JOIN",
    "SELECT",
    "WHERE",
    "FACET",
    "CASES",
    "COMPARE",
    "EXTRAPOLATE",
    "LIMIT",
    "OFFSET",
    "ORDER",
    "BY",
    "ASC",
    "DESC",
    "SINCE",
    "UNTIL",
    "AUTO",
    "PREDICT",
    "TIMEZONE",
    "SLIDE",
    "TIMESERIES",
    "METRIC_FORMAT",
    "RAW",
]


EXTENDED_KEYWORDS = [
    *KEYWORDS,
    "USING",
]


FUNCTIONS = [
    # Keywords: aggregator functions
    "aggregationEndTime",
    "apdex",
    "average",
    "bucketPercentile",
    "cardinality",
    "cdfPercentage",
    "count",
    "derivative",
    "earliest",
    "filter",
    "funnel",
    "histogram",
    "keyset",
    "latest",
    "latestrate",
    "max",
    "median",
    "min",
    "percentage",
    "percentile",
    "predictLinear",
    "rate",
    "stdvar",
    "sum",
    "uniqueCount",
    "uniques",
    # Keywords: non-aggregator functions
    "accountId",
    "aparse",
    "blob",
    "buckets",
    "capture",
    "cidrAddress",
    "concat",
    "contains",
    "convert",
    "decode",
    "dimensions",
    "encode",
    "eventType",
    "getCdfCount",
    "getField",
    "if",
    "jparse",
    "length",
    "lookup",
    "lower",
    "mapKeys",
    "mapValues",
    "mod",
    "position",
    "round",
    "stddev",
    "substring",
    "toDatetime",
    "toTimestamp",
    "upper",
    # Keywords: date and time functions
    "dateOf",
    "dayOfMonthOf",
    "daysInMonthOf",
    "hourOf",
    "minuteOf",
    "monthOf",
    "quarterOf",
    "weekdayOf",
    "weekOf",
    "yearOf",
    # Keywords: mathematical functions
    "abs",
    "ceil",
    "clamp_max",
    "clamp_min",
    "exp",
    "floor",
    "ln",
    "log",
    "log10",
    "log2",
    "pow",
    "sqrt",
    # Keywords: casting functions
    "boolean",
    "numeric",
    "string",
    "tuple",
]


OPERATORS = [
    r"*",
    r"=",
    r"+",
    r"-",
    r"/",
    r"%",
    r".",
    r"<",
    r">",
    r"<=",
    r">=",
    r"!=",
]


WORD_OPERATORS = [
    "AND",
    "AS",
    "IN",
    "IS",
    "LIKE",
    "NOT",
    "ON",
    "OR",
    "RLIKE",
]



# Autogenerated by nbdev

d = {
    "settings": {
        "branch": "master",
        "doc_baseurl": "/pygments-sb-nrql/",
        "doc_host": "https://dynovaio.github.io",
        "git_url": "https://github.com/dynova/pygments-sb-nrql/tree/master/",
        "lib_path": "src/pygments_sb_nrql",
    },
    "syms": {
        "pygments_sb_nrql.builtins": {},
        "pygments_sb_nrql.lexer": {
            "pygments_sb_nrql.lexer.NrqlLexer": (
                "lexer.html#nrqllexer",
                "pygments_sb_nrql/lexer.py",
            )
        },
        "pygments_sb_nrql.version": {},
    },
}
